      SUBROUTINE EVENT(NEV,P1,P2,FAC,IREJ)
C********************************************************************
C
C     main subroutine to manage simulation processes
C
C     input: NEV       -1   initialization
C                       1   generation of events
C                       2   generation of events without rejection
C                           due to energy dependent cross section
C                       3   generation of events without rejection
C                           due to initialization energy
C                      -2   output of event generation statistics
C            P1(4)     momentum of particle 1 (internal TARGET)
C            P2(4)     momentum of particle 2 (internal PROJECTILE)
C            FAC       used for initialization:
C                      contains cross section the events corresponds to
C                      during generation: current cross section
C
C     output: IREJ     0: event accepted
C                      1: event rejected
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           IZERO  =  0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           TINY   =  1.D-10)
C
      DIMENSION P1(4),P2(4)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
      COMMON /PROCES/ IPROCE,IDNODF,IDIFR1,IDIFR2,IDDPOM,IPRON(15)
C
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
C  global CMS system
      COMMON /GLOCMS/ ECM,PCM,PMASS(2),PVIRT(2),IFPAP(2),IFPAB(2)
C
      COMMON /XSECTP/ SIGTOT,SIGELA,SIGVM(0:4,0:4),SIGINE,SIGDIR,
     &                SIGLSD(2),SIGHSD(2),SIGLDD,SIGHDD,SIGCDF,
     &                SIGPOM,SIGREG,SIGHAR,SIGTR1,SIGTR2,SIGLOO,SIGDPO,
     &                SIG1SO,SIG1HA,SLOEL,SLOVM(4,4),SIGGEN(4),
     &                FSUP(2)
C
      PARAMETER ( MAXPRO = 16 )
      CHARACTER*18 PROC
      COMMON /PEPROC/ PROC(0:MAXPRO)
      INTEGER MXSECT
      PARAMETER ( MAXTAB = 20 )
      COMMON /HAXSEC/ XSECTA(4,-1:MAXPRO,4,MAXTAB),XSECT(6,-1:MAXPRO),
     &                MXSECT(0:4,-1:MAXPRO,4),ECMSH(4,MAXTAB),ISTTAB
      COMMON/POMRES/IPOPOS(2,100),IPORES(100),IPOIX1,IPOIX2
C
      DIMENSION IPRSAM(10),IPRACC(10),IENACC(10),IDNS(4),IDNA(4)
      DIMENSION XLIMB(2)
C
      IREJ = 0
C  initializations
      IF(NEV.EQ.-1) THEN
        WRITE(6,'(/3(/1X,A))')
     &    '=======================================================',
     &    '  ------- INITIALIZATION OF EVENT GENERATION --------',
     &    '======================================================='
        CALL TIMDAT
        CALL SETMDL(0,0,-2)
C  amplitude parameters
        CALL FITPAR(1)
        CALL NEWHIS(ZERO,ZERO,ZERO,XLIMB,-1,IREJ)
        CALL POHEPI(1,P1,P2,JM1,JM2)
        CALL REJECT(-1)
C  initialize MC package
        CALL POMCIN(JM1,JM2)
        CALL SAMPRO(-1)
        CALL PARTON(-1,0,0,P1,P2,IREJ)
C  initialize quark statistics
        CALL QUASTA(0,0,-1)
C  cross section
        FAC = SIGGEN(4)
        DO 20 I=1,10
          IPRSAM(I) = 0
          IPRACC(I) = 0
          IENACC(I) = 0
 20     CONTINUE
        ISPS = 0
        ISPA = 0
        ISRS = 0
        ISRA = 0
        IHPS = 0
        IHPA = 0
        ISTS = 0
        ISTA = 0
        ISLS = 0
        ISLA = 0
        IDIS = 0
        IDIA = 0
        IDPS = 0
        IDPA = 0
        IDNS(1) = 0
        IDNS(2) = 0
        IDNS(3) = 0
        IDNS(4) = 0
        IDNA(1) = 0
        IDNA(2) = 0
        IDNA(3) = 0
        IDNA(4) = 0
        KACCEP = 0
        KEVENT = 0
        KEVGEN = 0
        ECMSUM = ZERO
        CALL TIMDAT
      ELSE IF(NEV.GT.0) THEN
C
C  -------------- begin event generation ---------------
C
        IPAMDL(13) = 0
        IF(NEV.EQ.3) IPAMDL(13) = 1
        KEVENT = KEVENT+1
C  enable debugging
        CALL TRACE(0,0,0)
        IF(IDEB(68).GE.1) THEN
          IF((MOD(I,50).EQ.0).OR.(IDEB(68).GE.2))
     &      WRITE(6,'(1X,A,2I12)') 'CALL TO EVENT NO',KEVENT,KACCEP
        ENDIF
        CALL POHEPI(0,P1,P2,JM1,JM2)
C  cross section calculation
        FAC = SIGGEN(3)
        IF(NEV.EQ.1) THEN
          IF(RNDM(FAC).GT.(SIGGEN(3)/SIGGEN(4))) THEN
            IREJ = 1
            IF(IDEB(68).GE.3) WRITE(6,'(1X,A,I10,1P2E12.4)')
     &        'EVENT:DEBUG:REJECTION DUE TO CROSS SECTION',
     &        KEVENT,SIGGEN(3),SIGGEN(4)
            RETURN
          ENDIF
        ENDIF
        KEVGEN = KEVGEN+1
        SIGGEN(1) = SIGGEN(4)*DBLE(KEVGEN)/DBLE(KEVENT)
C
        ITRY1 = 0
 50     CONTINUE
          ITRY1 = ITRY1+1
          IF(ITRY1.GT.1) CALL POHEPI(1,P1,P2,JM1,JM2)
C  sample process
          IPROCE = 0
          CALL SAMPRO(IPROCE)
C  sampling statistics
          IPRSAM(IPROCE) = IPRSAM(IPROCE)+1
C
          ITRY2 = 0
 60       CONTINUE
            ITRY2 = ITRY2+1
            IF(ITRY2.GT.1) CALL POHEPI(1,P1,P2,JM1,JM2)
C  sample number of cut graphs according to IPROCE and
C  generate parton configurations+chains
            CALL PARTON(IPROCE,JM1,JM2,P1,P2,IREJ)
C  collect statistics
            ISPS = ISPS+KSPOM
            IHPS = IHPS+KHPOM
            ISRS = ISRS+KSREG
            ISTS = ISTS+KSTRG+KHTRG
            ISLS = ISLS+KSLOO+KHLOO
            IDIS = IDIS+MIN(KHDIR,1)
            IDPS = IDPS+KHDPO+KSDPO
            IF((IDIFR1+IDIFR2+IDDPOM.EQ.0).AND.(KHDIR.GT.0))
     &        IDNS(KHDIR) = IDNS(KHDIR)+1
C  rejection?
          IF(IREJ.NE.0) THEN
            IF((IREJ.EQ.50).AND.(NEV.EQ.1)) RETURN
            IFAIL(1) = IFAIL(1)+1
            IF(IDEB(68).GE.2) THEN
              WRITE(6,'(/1X,A,2I5)')
     &          'EVENT:WARNING:REJECTION BY PARTON',ITRY2,IREJ
              CALL POPREV(-1)
            ENDIF
            IF(ITRY1.GT.5) RETURN
            IF(IREJ.GE.5) THEN
              IF(ISWMDL(2).EQ.0) RETURN
              GOTO 50
            ENDIF
            IF(ITRY2.LT.5) GOTO 60
            GOTO 50
          ENDIF
C  fragmentation of chains
          CALL POFRAG(IREJ)
C  rejection?
          IF(IREJ.NE.0) THEN
            IFAIL(23) = IFAIL(23)+1
            IF(IDEB(68).GE.2)  THEN
              WRITE(6,'(/1X,A,2I5)')
     &          'EVENT:WARNING:REJECTION BY POFRAG',ITRY2,IREJ
              CALL POPREV(-1)
            ENDIF
            GOTO 50
          ENDIF
C  check of conservation of quantum numbers
          IF(IDEB(68).GE.-5) THEN
            CALL POQCHK(-1,IREJ)
            IF(IREJ.NE.0) GOTO 50
          ENDIF
C  event now completely processed and accepted
C  acceptance statistics
          IPRACC(IPROCE) = IPRACC(IPROCE)+1
          ISPA = ISPA+KSPOM
          IHPA = IHPA+KHPOM
          ISRA = ISRA+KSREG
          ISTA = ISTA+(KSTRG+KHTRG)
          ISLA = ISLA+(KSLOO+KHLOO)
          IDIA = IDIA+MIN(KHDIR,1)
          IDPA = IDPA+KHDPO+KSDPO
          IF((IDIFR1+IDIFR2.EQ.0).AND.(KHDIR.GT.0))
     &      IDNA(KHDIR) = IDNA(KHDIR)+1
          DO 55 I=1,IPOIX2
            IENACC(IPORES(I)) = IENACC(IPORES(I))+1
 55       CONTINUE
C  debug output
          IF(IDEB(68).GE.20) THEN
            IF(IDEB(68).GE.25) THEN
              CALL POPREV(1)
            ELSE
              CALL POPREV(0)
            ENDIF
          ENDIF
C  effective cross section
          KACCEP = KACCEP+1
          SIGGEN(2) = SIGGEN(4)*DBLE(KACCEP)/DBLE(KEVENT)
          ECMSUM = ECMSUM+ECM
      ELSE IF(NEV.EQ.-2) THEN
C
C  ---------------- end of event generation ----------------------
C
        WRITE(6,'(/3(/1X,A),//1X,A,3I12,/1X,A,F12.1)')
     &    '====================================================',
     &    '  --------- SUMMARY OF EVENT GENERATION ----------',
     &    '====================================================',
     &    'CALLED,GENERATED,ACCEPTED EVENTS:',KEVENT,KEVGEN,KACCEP,
     &    'AVERAGE CMS ENERGY:',ECMSUM/DBLE(MAX(1,KACCEP))
        CALL TIMDAT
C  write out statistics
        IF(KACCEP.GT.0) THEN
C
          FAC1 = SIGGEN(4)/DBLE(KEVENT)
          FAC2 = FAC/DBLE(KACCEP)
          WRITE(6,'(/1X,A,/1X,A)')
     &      'EVENT:DEBUG:GENERATED AND ACCEPTED EVENTS:',
     &      '========================================='
          WRITE(6,'(3X,A)')
     &   'PROCESS, SAMPLED, ACCEPTED, CROSS SECTION (INTERNAL/EXTERNAL)'
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'NON.DIFF.',IPRSAM(1),
     &      IPRACC(1),DBLE(IPRACC(1))*FAC1,DBLE(IPRACC(1))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'ELAS SCA.',IPRSAM(2),
     &      IPRACC(2),DBLE(IPRACC(2))*FAC1,DBLE(IPRACC(2))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'QELA SCA.',IPRSAM(3),
     &      IPRACC(3),DBLE(IPRACC(3))*FAC1,DBLE(IPRACC(3))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DOUB.POM.',IPRSAM(4),
     &      IPRACC(4),DBLE(IPRACC(4))*FAC1,DBLE(IPRACC(4))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DIFF.PAR1',IPRSAM(5),
     &      IPRACC(5),DBLE(IPRACC(5))*FAC1,DBLE(IPRACC(5))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DIFF.PAR2',IPRSAM(6),
     &      IPRACC(6),DBLE(IPRACC(6))*FAC1,DBLE(IPRACC(6))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DOUB.DIF.',IPRSAM(7),
     &      IPRACC(7),DBLE(IPRACC(7))*FAC1,DBLE(IPRACC(7))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DIR ALL  ',IPRSAM(8),
     &      IPRACC(8),DBLE(IPRACC(8))*FAC1,DBLE(IPRACC(8))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DIR x RES',IDNS(1),IDNA(1),
     &      DBLE(IDNA(1))*FAC1,DBLE(IDNA(1))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'RES x DIR',IDNS(2),IDNA(2),
     &      DBLE(IDNA(2))*FAC1,DBLE(IDNA(2))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DIR x DIR',IDNS(3),IDNA(3),
     &      DBLE(IDNA(3))*FAC1,DBLE(IDNA(3))*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'SOFT POM.',ISPS,ISPA,
     &      DBLE(ISPA)*FAC1,DBLE(ISPA)*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'HARD POM.',IHPS,IHPA,
     &      DBLE(IHPA)*FAC1,DBLE(IHPA)*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'SOFT REG.',ISRS,ISRA,
     &      DBLE(ISRA)*FAC1,DBLE(ISRA)*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'ENH. TRG.',ISTS,ISTA,
     &      DBLE(ISTA)*FAC1,DBLE(ISTA)*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'ENH. LOG.',ISLS,ISLA,
     &      DBLE(ISLA)*FAC1,DBLE(ISLA)*FAC2
          WRITE(6,'(3X,A,2I12,1P2E13.3)') 'DOUB.POM.',IDPS,IDPA,
     &      DBLE(IDPA)*FAC1,DBLE(IDPA)*FAC2
          WRITE(6,'(3X,A,I3)') 'recursive pomeron splitting:',ISWMDL(14)
          WRITE(6,'(5X,A,I12)') '1->2pom-cut :',IENACC(8)
          WRITE(6,'(5X,A,I12)') '1->doub-pom :',IENACC(4)
          WRITE(6,'(5X,A,I12)') '1->diff-dis1:',IENACC(5)
          WRITE(6,'(5X,A,I12)') '1->diff-dis2:',IENACC(6)
          WRITE(6,'(5X,A,I12)') '1->doub-diff:',IENACC(7)
          WRITE(6,'(2(/1X,A,1PE12.3)/)') ' SAMPLED CROSS SECTION (mb)',
     &      SIGGEN(1),'ACCEPTED CROSS SECTION (mb)',SIGGEN(2)
C
          CALL REJECT(-2)
          CALL SAMPRO(-2)
          CALL PARTON(-2,0,0,P1,P2,IREJ)
C  statistics of hard scattering processes
          WRITE(6,'(2(/1X,A))') 
     &      'EVENT:DEBUG:STATISTICS OF HARD SCATTERING PROCESSES',
     &      '==================================================='
          DO 43 K=1,4
            IF(MXSECT(1,0,K).GT.0) THEN
              WRITE(6,'(5X,A,I3)')
     &      'PROCESS (ACCEPTED,X-SECTION INTERNAL/EXTERNAL) FOR IP:',K
              DO 47 M=0,MAXPRO
                WRITE(6,'(1X,I3,1X,A,2X,2I12,1P2E13.3)') M,PROC(M),
     &          MXSECT(1,M,K),MXSECT(2,M,K),DBLE(MXSECT(2,M,K))*FAC1,
     &            DBLE(MXSECT(2,M,K))*FAC2
 47           CONTINUE
            ENDIF
 43       CONTINUE
C
          IF(IDEB(68).GE.1) CALL QUASTA(0,0,-2)
          IF(IDEB(68).GE.2) CALL PARTPT(-2,0,0,0.D0,0)
        ELSE
          WRITE(6,'(/1X,A,I4,/)') 'NO OUTPUT OF STATISTICS',KEVENT
        ENDIF
        WRITE(6,'(/3(/1X,A)/)')
     &    '======================================================',
     &    '   ------- END OF EVENT GENERATION SUMMARY --------',
     &    '======================================================'
      ELSE
        WRITE(6,'(/1X,A,I7)') 'EVENT:ERROR:UNSUPPORTED NEV',NEV
      ENDIF
      END
