C
C
      SUBROUTINE GGBLSR(NEVENT,EE1,EE2)
C**********************************************************************
C
C     interface to call PHOJET (variable energy run) for
C     gamma-gamma collisions via laser backscattering
C
C     input:     EE1     LAB system energy of electron/positron 1
C                EE2     LAB system energy of electron/positron 2
C                NEVENT  number of events to generate
C            from COMMON /LEPCUT/:
C                YMIN1   lower limit of Y1
C                        (energy fraction taken by photon from electron)
C                YMAX1   upper limit of Y1
C                YMIN2   lower limit of Y2
C                        (energy fraction taken by photon from electron)
C                YMAX2   upper limit of Y2
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( ZERO = 0.D0,
     &            ONE  = 1.D0,
     &            TWO  = 2.D0,
     &            PI   = 3.14159265359D0 )
C
C  lepton/hadron-photon kinematics
      COMMON /PHOSRC/ PINI(5,2),PFIN(5,2),PGAM(5,2),
     &                GYY(2),GQ2(2),GGECM,GAIMP(2),PFTHE(2),PFPHI(2),
     &                IDPSRC(2),IDBSRC(2),RADSRC(2),AMSRC(2),GAMSRC(2)
      COMMON /LEPCUT/ ECMIN,ECMAX,EEMIN1,EEMIN2,
     &                YMIN1,YMAX1,YMIN2,YMAX2,
     &                Q2MIN1,Q2MAX1,Q2MIN2,Q2MAX2,
     &                THMIN1,THMAX1,THMIN2,THMAX2
C
      DIMENSION P1(4),P2(4)
      CHARACTER*72 TITLE
C
      WRITE(6,'(//1X,A,I10)') 'GGBLSR:DEBUG:EVENTS TO PROCESS',NEVENT
C
      YMAX1 = MIN(0.828D0,YMAX1)
      YMAX2 = MIN(0.828D0,YMAX2)
      IF((YMIN1.GT.0.827D0).OR.(YMIN2.GT.0.827D0)) THEN
        WRITE(6,'(/1X,A,2E12.4)') 'GGBLSR:ERROR:INVALID YMIN1,YMIN2',
     &    YMIN1,YMIN2
        RETURN
      ENDIF
      IDPSRC(1) = 0
      IDBSRC(1) = 0
      IDPSRC(2) = 0
      IDBSRC(2) = 0
C
      MAXTAB = 50
*     WRITE(6,'(1X,A,I5)')
*    &  'GGBLSR:DEBUG:TABLE OF PHOTON FLUX (SIDE 1)',MAXTAB
      DELY = (YMAX1-YMIN1)/DBLE(MAXTAB-1)
      DO 100 I=1,MAXTAB
        Y1 = YMIN1+DELY*DBLE(I-1)
        FF = (-0.544*Y1**3+2.17*Y1**2-2.63*Y1+1.09)/(1-Y1)**2
*       WRITE(6,'(5X,2E14.4)') Y1,FF
 100  CONTINUE
C
      FLUX1A = 1.021D0*(YMAX1-YMIN1)
      FLUX1B = 0.07D0/(ONE-YMAX1)-0.07D0/(ONE-YMIN1)
      FLUX2A = 1.021D0*(YMAX2-YMIN2)
      FLUX2B = 0.07D0/(ONE-YMAX2)-0.07D0/(ONE-YMIN2)
      FLUX = (FLUX1A+FLUX1B)*(FLUX2A+FLUX2B)
C
      PR1 = FLUX1A/(FLUX1A+FLUX1B)
      PR2 = FLUX2A/(FLUX2A+FLUX2B)
      XID1A = YMAX1-YMIN1
      XIM1A = YMIN1
      XID1B = ONE/(ONE-YMAX1)-ONE/(ONE-YMIN1)
      XIM1B = ONE/(ONE-YMIN1)
      XID2A = YMAX2-YMIN2
      XIM2A = YMIN2
      XID2B = ONE/(ONE-YMAX2)-ONE/(ONE-YMIN2)
      XIM2B = ONE/(ONE-YMIN2)
C
C  photon 1
      EGAM = YMAX1*EE1
      P1(1) = ZERO
      P1(2) = ZERO
      P1(3) = EGAM
      P1(4) = EGAM
C  photon 2
      EGAM = YMAX2*EE2
      P2(1) = ZERO
      P2(2) = ZERO
      P2(3) = -EGAM
      P2(4) = EGAM
      CALL SETPAR(1,22,0,ZERO)
      CALL SETPAR(2,22,0,ZERO)
      CALL EVENT(-1,P1,P2,SIGMAX,IREJ)
      CALL POHIST(-1,WEIGHT)
      CALL POLUHI(-1,WEIGHT)
C
C  generation of events
      ECUP = 2.D0*SQRT(YMAX1*YMAX2*EE1*EE2)*1.1D0
      ECUPL = LOG(ECUP)
      ECMINL = LOG(ECMIN)
      CALL NEWHIS(ECMINL,ECUPL,ZERO,P1,49,IGGLUM)
      CALL NEWHIS(YMIN1,YMAX1,ZERO,P1,49,IHFLX1)
      CALL NEWHIS(YMIN2,YMAX2,ZERO,P1,49,IHFLX2)
      CALL NEWHIS(YMIN1,YMAX1,ZERO,P1,49,IHFLY1)
      CALL NEWHIS(YMIN2,YMAX2,ZERO,P1,49,IHFLY2)
      AY1  = ZERO
      AY2  = ZERO
      AYS1 = ZERO
      AYS2 = ZERO
      NITER = NEVENT
      ITRY = 0
      ITRW = 0
      DO 200 I=1,NITER
 150    CONTINUE
        ITRY = ITRY+1
 175    CONTINUE
          ITRW = ITRW+1
          IF(RNDM(AY1).LT.PR1) THEN
            Y1 = XID1A*RNDM(AY2)+XIM1A
          ELSE
            Y1 = XID1B*RNDM(AYS1)+XIM1B
            Y1 = ONE-ONE/Y1
          ENDIF
          IF(RNDM(AYS2).LT.PR2) THEN
            Y2 = XID2A*RNDM(Y1)+XIM2A
          ELSE
            Y2 = XID2B*RNDM(Y2)+XIM2B
            Y2 = ONE-ONE/Y2
          ENDIF
          WG1 = (-0.544*Y1**3+2.17*Y1**2-2.63*Y1+1.09)/(1-Y1)**2
          WG2 = (-0.544*Y2**3+2.17*Y2**2-2.63*Y2+1.09)/(1-Y2)**2
          WGH = WG1*WG2/(1.021+0.07/(1-Y1)**2)/(1.021+0.07/(1-Y2)**2)
          IF(WGH.GT.ONE) THEN
            WRITE(6,'(1X,A,5E11.4)')
     &        'GGBLSR:WEIGHT ERROR:',Y1,Y2,WGH,WG1,WG2
          ENDIF
        IF(RNDM(WGH).GT.WGH) GOTO 175
        CALL FILHIS(Y1,ONE,IHFLX1)
        CALL ADDHIS(IHFLX1)
        CALL FILHIS(Y2,ONE,IHFLX2)
        CALL ADDHIS(IHFLX2)
        Q2P1 = ZERO
        Q2P2 = ZERO
        GYY(1) = Y1
        GQ2(1) = Q2P1
        GYY(2) = Y2
        GQ2(2) = Q2P2
C  incoming electron 1
        PINI(1,1) = ZERO
        PINI(2,1) = ZERO
        PINI(3,1) = EE1
        PINI(4,1) = EE1
        PINI(5,1) = ZERO
C  outgoing electron 1
        YQ2 = SQRT((ONE-Y1)*Q2P2)
        Q2E = Q2P1/(4.D0*EE1)
        E1Y = EE1*(ONE-Y1)
        CALL SFECFE(SIF,COF)
        PFIN(1,1) = YQ2*COF
        PFIN(2,1) = YQ2*SIF
        PFIN(3,1) = E1Y-Q2E
        PFIN(4,1) = E1Y+Q2E
        PFIN(5,1) = ZERO
C  photon 1
        P1(1) = -PFIN(1,1)
        P1(2) = -PFIN(2,1)
        P1(3) = PINI(3,1)-PFIN(3,1)
        P1(4) = PINI(4,1)-PFIN(4,1)
C  incoming electron 2
        PINI(1,2) = ZERO
        PINI(2,2) = ZERO
        PINI(3,2) = -EE2
        PINI(4,2) = EE2
        PINI(5,2) = ZERO
C  outgoing electron 2
        YQ2 = SQRT((ONE-Y2)*Q2P2)
        Q2E = Q2P2/(4.D0*EE2)
        E1Y = EE2*(ONE-Y2)
        CALL SFECFE(SIF,COF)
        PFIN(1,2) = YQ2*COF
        PFIN(2,2) = YQ2*SIF
        PFIN(3,2) = -E1Y+Q2E
        PFIN(4,2) = E1Y+Q2E
        PFIN(5,2) = ZERO
C  photon 2
        P2(1) = -PFIN(1,2)
        P2(2) = -PFIN(2,2)
        P2(3) = PINI(3,2)-PFIN(3,2)
        P2(4) = PINI(4,2)-PFIN(4,2)
C  ECMS cut
        GGECM = (P1(4)+P2(4))**2-(P1(1)+P2(1))**2
     &         -(P1(2)+P2(2))**2-(P1(3)+P2(3))**2
        IF(GGECM.LT.0.1D0) GOTO 175
        GGECM = SQRT(GGECM)
        IF((GGECM.LT.ECMIN).OR.(GGECM.GT.ECMAX)) GOTO 175
C
        PGAM(1,1) = P1(1)
        PGAM(2,1) = P1(2)
        PGAM(3,1) = P1(3)
        PGAM(4,1) = P1(4)
        PGAM(5,1) = ZERO
        PGAM(1,2) = P2(1)
        PGAM(2,2) = P2(2)
        PGAM(3,2) = P2(3)
        PGAM(4,2) = P2(4)
        PGAM(5,2) = ZERO
        CALL PRESEL(5,IREJ)
        IF(IREJ.NE.0) GOTO 175
C  event generation
        CALL EVENT(1,P1,P2,SIGCUR,IREJ)
        IF(IREJ.NE.0) GOTO 150
	GGECML = LOG(GGECM)
        CALL FILHIS(GGECML,ONE,IGGLUM)
        CALL ADDHIS(IGGLUM)
        CALL FILHIS(Y1,ONE,IHFLY1)
        CALL ADDHIS(IHFLY1)
        CALL FILHIS(Y2,ONE,IHFLY2)
        CALL ADDHIS(IHFLY2)
C  statistics
        AY1  = AY1+Y1
        AYS1 = AYS1+Y1*Y1
        AY2  = AY2+Y2
        AYS2 = AYS2+Y2*Y2
C  histograms
        CALL POHIST(1,ONE)
        CALL POLUHI(1,ONE)
 200  CONTINUE
C
      WGY  = FLUX*DBLE(ITRY)/DBLE(ITRW)
      AY1  = AY1/DBLE(NITER)
      AYS1 = AYS1/DBLE(NITER)
      DAY1 = SQRT((AYS1-AY1**2)/DBLE(NITER))
      AY2  = AY2/DBLE(NITER)
      AYS2 = AYS2/DBLE(NITER)
      DAY2 = SQRT((AYS2-AY2**2)/DBLE(NITER))
      WEIGHT = WGY*SIGMAX*DBLE(NITER)/DBLE(ITRY)
C  output of statistics, histograms
      WRITE(6,'(//1X,A,/1X,A,1PE12.3,A,/1X,A)')
     &'=========================================================',
     &' *****   simulated cross section: ',WEIGHT,' mb  *****',
     &'========================================================='
      WRITE(6,'(//1X,A,3I10)')
     &  'GGBLSR:SUMMARY:NITER,ITRY,ITRW',NITER,ITRY,ITRW
      WRITE(6,'(1X,A,1P2E12.4)') 'EFFECTIVE WEIGHT (FLUX,TOTAL)',
     &  WGY,WEIGHT
      WRITE(6,'(1X,A,2F10.5)') 'GGBLSR:AVERAGE Y1,DY1 ',AY1,DAY1
      WRITE(6,'(1X,A,2F10.5)') 'GGBLSR:AVERAGE Y2,DY2 ',AY2,DAY2
C
      CALL EVENT(-2,P1,P2,WEIGHT,IREJ)
      IF(NITER.GT.1) THEN
        CALL POHIST(-2,WEIGHT)
        CALL POLUHI(-2,WEIGHT)
      ELSE
        WRITE(6,'(1X,A,I4)') 'GGBLSR:NO OUTPUT OF HISTOGRAMS',NITER
      ENDIF
C
      TITLE = ' LASER BS PHOTON-PHOTON LUMINOSITY DPHI/D(ECM-GG)'
      FAC =  WGY
      CALL OUTHIS(IGGLUM,3,TITLE,FAC,1)
      FAC = SQRT(WGY)
      TITLE = ' BACKSCATTERED LASER PHOTON FLUX SIDE 1'
      CALL OUTHIS(IHFLX1,1,TITLE,FAC,1)
      TITLE = ' BACKSCATTERED LASER PHOTON FLUX SIDE 2'
      CALL OUTHIS(IHFLX2,1,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 1 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY1,1,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 2 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY2,1,TITLE,FAC,1)
      END
