C
C
      SUBROUTINE GGHIOG(NEVENT,EEN,NA,NZ)
C**********************************************************************
C
C     interface to call PHOJET (variable energy run) for
C     gamma-gamma collisions via heavy ions (geometrical approach)
C
C
C     input:     EEN     LAB system energy per nucleon
C                NA      atomic number of ion/hadron
C                NZ      charge number of ion/hadron
C                NEVENT  number of events to generate
C            from COMMON /LEPCUT/:
C                YMIN1,2 lower limit of Y
C                        (energy fraction taken by photon from hadron)
C                YMAX1,2 upper cutoff for Y, necessary to avoid
C                        underflows
C
C      currently implemented approximation similar to:
C                
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( ZERO = 0.D0,
     &            ONE  = 1.D0,
     &            OHALF= 0.5D0,
     &            TWO  = 2.D0,
     &            THREE= 3.D0,
     &            DEPS = 1.D-20,
     &            PI   = 3.14159265359D0 )
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
C  lepton/hadron-photon kinematics
      COMMON /PHOSRC/ PINI(5,2),PFIN(5,2),PGAM(5,2),
     &                GYY(2),GQ2(2),GGECM,GAIMP(2),PFTHE(2),PFPHI(2),
     &                IDPSRC(2),IDBSRC(2),RADSRC(2),AMSRC(2),GAMSRC(2)
      COMMON /LEPCUT/ ECMIN,ECMAX,EEMIN1,EEMIN2,
     &                YMIN1,YMAX1,YMIN2,YMAX2,
     &                Q2MIN1,Q2MAX1,Q2MIN2,Q2MAX2,
     &                THMIN1,THMAX1,THMIN2,THMAX2
C
      PARAMETER (MAXTAB=100)
      DIMENSION P1(4),P2(4),TABCU(0:MAXTAB),TABYL(0:MAXTAB)
      CHARACTER*72 TITLE
C
      WRITE(6,'(2(/1X,A))') 'GGHIOG:GAMMA-GAMMA EVENT GENERATION',
     &                      '==================================='
C  hadron size and mass
      FM2GEV = 5.07D0
      HIMASS = DBLE(NA)*0.938D0
      HIMA2  = HIMASS**2
      HIRADI = 1.2D0*FM2GEV*DBLE(NA)**0.333
      ALPHA  = DBLE(NZ**2)/137.D0
C  total hadron / heavy ion energy
      EE     = EEN*DBLE(NA)
      GAMMA  = EE/HIMASS
C  setup /PHOSRC/
      GAMSRC(1) = GAMMA
      GAMSRC(2) = GAMMA
      RADSRC(1) = HIRADI
      RADSRC(2) = HIRADI
      AMSRC(1)  = HIMASS
      AMSRC(1)  = HIMASS
C  kinematic limitations
      YMI = (ECMIN/(2.D0*EE))**2
      IF(YMIN1.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)') 
     &    'GGHIOG:DEBUG:YMIN1 INCREASED TO (OLD/NEW)',YMIN1,YMI
        YMIN1 = YMI
      ELSE IF(YMIN1.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGHIOG:WARNING:','ECM-CUT CORRESPONDS TO YMIN1 OF',YMI,
     &    '  INSTEAD OF',YMIN1
      ENDIF
      IF(YMIN2.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)') 
     &    'GGHIOG:DEBUG:YMIN2 INCREASED TO (OLD/NEW)',YMIN2,YMI
        YMIN2 = YMI
      ELSE IF(YMIN2.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGHIOG:WARNING:','ECM-CUT CORRESPONDS TO YMIN2 OF',YMI,
     &    '  INSTEAD OF',YMIN2
      ENDIF
C  debug output
      WRITE(6,'(/6X,A,2I4)')   'MASS NUMBER, CHARGE NUMBER  ',NA,NZ
      WRITE(6,'(6X,A,E12.5)') 'HADRON MASS (GeV)           ',HIMASS
      WRITE(6,'(6X,A,E12.5)') 'HADRON RADIUS (GeV**-1)     ',HIRADI
      WRITE(6,'(6X,A,E12.5)') 'LORENTZ GAMMA               ',GAMMA
      WRITE(6,'(6X,A,2E12.5)') 'Y RANGE PHOTON 1            ',YMIN1,
     &  YMAX1
      WRITE(6,'(6X,A,2E12.5)') 'Y RANGE PHOTON 2            ',YMIN2,
     &  YMAX2
      WRITE(6,'(6X,A,2E12.5)') 'SQRT(S) PER NUCLEON, TOTAL  ',
     &  2.D0*EEN,2.D0*EE
      WRITE(6,'(6X,A,I10)')   'EVENTS TO PROCESS           ',NEVENT
C  hadron numbers set to 0
      IDPSRC(1) = 0
      IDBSRC(1) = 0
      IDPSRC(2) = 0
      IDBSRC(2) = 0
C  table of flux function, log interpolation
      YMIN = YMIN1
      YMAX = YMAX1
      YMAX = MIN(YMAX,0.9999999D0)
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      TABYL(0) = LOG(YMIN)
      FFMAX = ZERO
      DO 100 I=1,MAXTAB
        Y = EXP(TABYL(0)+DELLY*DBLE(I-1))
        WG = EE*Y
        XI = WG*HIRADI/GAMMA
        FF = ALPHA*GGFLCL(XI)/Y
        FFMAX = MAX(FF,FFMAX)
        IF(FF.LT.1.D-10*FFMAX) THEN
          WRITE(6,'(/1X,A,2E12.4)')
     &      'GGHIOG:DEBUG:YMAX1 CHANGED FROM/TO',YMAX1,Y
          YMAX1 = MIN(Y,YMAX1)
          GOTO 101
        ENDIF
 100  CONTINUE
 101  CONTINUE
      YMIN = YMIN2
      YMAX = YMAX2
      YMAX = MIN(YMAX,0.9999999D0)
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      TABYL(0) = LOG(YMIN)
      FFMAX = ZERO
      DO 102 I=1,MAXTAB
        Y = EXP(TABYL(0)+DELLY*DBLE(I-1))
        WG = EE*Y
        XI = WG*HIRADI/GAMMA
        FF = ALPHA*GGFLCL(XI)/Y
        FFMAX = MAX(FF,FFMAX)
        IF(FF.LT.1.D-10*FFMAX) THEN
          WRITE(6,'(/1X,A,2E12.4)')
     &      'GGHIOG:DEBUG:YMAX2 CHANGED FROM/TO',YMAX2,Y
          YMAX2 = MIN(Y,YMAX2)
          GOTO 103
        ENDIF
 102  CONTINUE
 103  CONTINUE
      YMI = (ECMIN/(2.D0*EE))**2/YMAX2
      IF(YMI.GT.YMIN1) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOG:DEBUG:YMIN1 CHANGED FROM/TO',YMIN1,YMI
        YMIN1 = YMI
      ENDIF
      YMAX1 = MIN(YMAX,YMAX1)
      YMI = (ECMIN/(2.D0*EE))**2/YMAX1
      IF(YMI.GT.YMIN2) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOG:DEBUG:YMIN2 CHANGED FROM/TO',YMIN2,YMI
        YMIN2 = YMI
      ENDIF
C
      YMIN = YMIN1
      YMAX = YMAX1
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      TABCU(0) = ZERO
      TABYL(0) = LOG(YMIN)
      FLUX = ZERO
*     WRITE(6,'(/1X,A,I5)') 
*    &  'GGHIOG:DEBUG:TABLE OF RAW PHOTON FLUX (SIDE 1)',MAXTAB
      DO 105 I=1,MAXTAB
        Y = EXP(TABYL(0)+DELLY*DBLE(I-1))
        WG = EE*Y
        XI = WG*HIRADI/GAMMA
        FF = ALPHA*GGFLCL(XI)/Y
        FFMAX = MAX(FF,FFMAX)
        TABCU(I) = TABCU(I-1)+FF*Y
        TABYL(I) = LOG(Y)
        FLUX = FLUX+Y*FF
*       WRITE(6,'(5X,2E15.4)') Y,FF
 105  CONTINUE
      FLUX = FLUX*DELLY
      WRITE(6,'(1X,A,E12.4)') 
     &  'GGHIOG:DEBUG:INTEGRATED FLUX (ONE SIDE):',FLUX
C
C  initialization
C  photon 1
      EGAM = YMAX*EE
      P1(1) = ZERO
      P1(2) = ZERO
      P1(3) = EGAM
      P1(4) = EGAM
C  photon 2
      EGAM = YMAX*EE
      P2(1) = ZERO
      P2(2) = ZERO
      P2(3) = -EGAM
      P2(4) = EGAM
      CALL SETPAR(1,22,0,ZERO)
      CALL SETPAR(2,22,0,ZERO)
      CALL EVENT(-1,P1,P2,SIGMAX,IREJ)
      CALL POHIST(-1,WEIGHT)
      CALL POLUHI(-1,WEIGHT)
C
C  generation of events
      ECUP = 2.D0*SQRT(YMAX*YMAX*EE*EE)*1.1D0
      YMINL = LOG(ECMIN)
      YMAXL = LOG(ECUP)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IGGLUM)
      YMINL = LOG(YMIN1)
      YMAXL = LOG(YMAX1)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLX1)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLY1)
      YMINL = LOG(YMIN2)
      YMAXL = LOG(YMAX2)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLX2)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLY2)
      AY1  = ZERO
      AY2  = ZERO
      AYS1 = ZERO
      AYS2 = ZERO
      YY1MIN = 1.D30
      YY2MIN = 1.D30
      YY1MAX = 0.D0
      YY2MAX = 0.D0
      NITER = NEVENT
      ITRY = 0
      ITRW = 0
      DO 200 I=1,NITER
 150    CONTINUE
        ITRY = ITRY+1
 175    CONTINUE
        ITRW = ITRW+1
        XI = RNDM(AY1)*TABCU(MAXTAB)
        DO 110 K=1,MAXTAB
          IF(TABCU(K).GE.XI) THEN
            Y1 = DELLY/(TABCU(K)-TABCU(K-1))*(XI-TABCU(K-1))+TABYL(K-1)
            Y1 = EXP(Y1)
            GOTO 120
          ENDIF
 110    CONTINUE
        Y1 = YMAX1
 120    CONTINUE
        XI = RNDM(AY2)*TABCU(MAXTAB)
        DO 130 K=1,MAXTAB
          IF(TABCU(K).GE.XI) THEN
            Y2 = DELLY/(TABCU(K)-TABCU(K-1))*(XI-TABCU(K-1))+TABYL(K-1)
            Y2 = EXP(Y2)
            GOTO 140
          ENDIF
 130    CONTINUE
        Y2 = YMAX2
 140    CONTINUE
C  setup kinematics
        Y1L = LOG(Y1)
        Y2L = LOG(Y2)
        CALL FILHIS(Y1L,ONE,IHFLX1)
        CALL ADDHIS(IHFLX1)
        CALL FILHIS(Y2L,ONE,IHFLX2)
        CALL ADDHIS(IHFLX2)
        GYY(1) = Y1
        GQ2(1) = ZERO
        GYY(2) = Y2
        GQ2(2) = ZERO
C  incoming electron 1
        PINI(1,1) = ZERO
        PINI(2,1) = ZERO
        PINI(3,1) = EE
        PINI(4,1) = EE
        PINI(5,1) = ZERO
C  outgoing electron 1
        E1Y = EE*(ONE-Y1)
        PFIN(1,1) = ZERO
        PFIN(2,1) = ZERO
        PFIN(3,1) = E1Y
        PFIN(4,1) = E1Y
        PFIN(5,1) = ZERO
C  photon 1
        P1(1) = -PFIN(1,1)
        P1(2) = -PFIN(2,1)
        P1(3) = PINI(3,1)-PFIN(3,1)
        P1(4) = PINI(4,1)-PFIN(4,1)
C  incoming electron 2
        PINI(1,2) = ZERO
        PINI(2,2) = ZERO
        PINI(3,2) = -EE
        PINI(4,2) = EE
        PINI(5,2) = ZERO
C  outgoing electron 2
        E1Y = EE*(ONE-Y2)
        PFIN(1,2) = ZERO
        PFIN(2,2) = ZERO
        PFIN(3,2) = -E1Y
        PFIN(4,2) = E1Y
        PFIN(5,2) = ZERO
C  photon 2
        P2(1) = -PFIN(1,2)
        P2(2) = -PFIN(2,2)
        P2(3) = PINI(3,2)-PFIN(3,2)
        P2(4) = PINI(4,2)-PFIN(4,2)
C  ECMS cut
        GGECM = (P1(4)+P2(4))**2-(P1(3)+P2(3))**2
        IF(GGECM.LT.0.1D0) GOTO 175
        GGECM = SQRT(GGECM)
        IF((GGECM.LT.ECMIN).OR.(GGECM.GT.ECMAX)) GOTO 175
        PGAM(1,1) = P1(1)
        PGAM(2,1) = P1(2)
        PGAM(3,1) = P1(3)
        PGAM(4,1) = P1(4)
        PGAM(5,1) = ZERO
        PGAM(1,2) = P2(1)
        PGAM(2,2) = P2(2)
        PGAM(3,2) = P2(3)
        PGAM(4,2) = P2(4)
        PGAM(5,2) = ZERO
C  impact parameter constraints
        XI1   = P1(4)*HIRADI/GAMMA
        XI2   = P2(4)*HIRADI/GAMMA
        FLX   = GGFLCL(XI1)*GGFLCL(XI2)
        FCORR = GGFLCR(HIRADI)
        WGX   = (FLX-FCORR)/FLX
        IF(RNDM(Y2).GT.WGX) GOTO 175
C  cut given by user
        CALL PRESEL(5,IREJ)
        IF(IREJ.NE.0) GOTO 175
C  event generation
        CALL EVENT(1,P1,P2,SIGCUR,IREJ)
        IF(IREJ.NE.0) GOTO 150
        GGECML = LOG(GGECM)
        CALL FILHIS(GGECML,ONE,IGGLUM)
        CALL ADDHIS(IGGLUM)
        CALL FILHIS(Y1L,ONE,IHFLY1)
        CALL ADDHIS(IHFLY1)
        CALL FILHIS(Y2L,ONE,IHFLY2)
        CALL ADDHIS(IHFLY2)
C  statistics
        AY1  = AY1+Y1
        AYS1 = AYS1+Y1*Y1
        AY2  = AY2+Y2
        AYS2 = AYS2+Y2*Y2
        YY1MIN = MIN(YY1MIN,Y1)
        YY2MIN = MIN(YY2MIN,Y2)
        YY1MAX = MAX(YY1MAX,Y1)
        YY2MAX = MAX(YY2MAX,Y2)
C  histograms
        CALL POHIST(1,ONE)
        CALL POLUHI(1,ONE)
 200  CONTINUE
C
      WGY  = FLUX**2*DBLE(ITRY)/DBLE(ITRW)
      AY1  = AY1/DBLE(NITER)
      AYS1 = AYS1/DBLE(NITER)
      DAY1 = SQRT((AYS1-AY1**2)/DBLE(NITER))
      AY2  = AY2/DBLE(NITER)
      AYS2 = AYS2/DBLE(NITER)
      DAY2 = SQRT((AYS2-AY2**2)/DBLE(NITER))
      WEIGHT = WGY*SIGMAX*DBLE(NITER)/DBLE(ITRY)
C  output of statistics, histograms
      WRITE(6,'(//1X,A,/1X,A,1PE12.3,A,/1X,A)')
     &'=========================================================',
     &' *****   simulated cross section: ',WEIGHT,' mb  *****',
     &'========================================================='
      WRITE(6,'(//1X,A,3I12)')
     &  'GGHIOG:SUMMARY:NITER,ITRY,ITRW',NITER,ITRY,ITRW
      WRITE(6,'(1X,A,1P2E12.4)') 'EFFECTIVE WEIGHT (FLUX,TOTAL)',
     &  WGY,WEIGHT
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y1,DY1               ',
     &  AY1,DAY1
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y2,DY2               ',
     &  AY2,DAY2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 1     ',
     &  YY1MIN,YY1MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 2     ',
     &  YY2MIN,YY2MAX

C
      CALL EVENT(-2,P1,P2,WEIGHT,IREJ)
      IF(NITER.GT.1) THEN
        CALL POHIST(-2,WEIGHT)
        CALL POLUHI(-2,WEIGHT)
      ELSE
        WRITE(6,'(1X,A,I4)') 'GGHIOG:NO OUTPUT OF HISTOGRAMS',NITER
      ENDIF
C
      TITLE = ' HEAVY ION PHOTON-PHOTON LUMINOSITY DPHI/D(ECM-GG)'
      CALL OUTHIS(IGGLUM,3,TITLE,WGY,1)
      TITLE = ' HEAVY ION PHOTON-PHOTON CROSS SECTION DSIG/D(ECM-GG)'
      CALL OUTHIS(IGGLUM,3,TITLE,WEIGHT,1)
      TITLE = ' HEAVY ION PHOTON FLUX (SEMI-CLASS. APPROACH) SIDE 1'
      CALL OUTHIS(IHFLX1,3,TITLE,FLUX,1)
      TITLE = ' HEAVY ION PHOTON FLUX (SEMI-CLASS. APPROACH) SIDE 2'
      CALL OUTHIS(IHFLX2,3,TITLE,FLUX,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 1 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY1,3,TITLE,FLUX,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 2 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY2,3,TITLE,FLUX,1)
      END
