C
C
      SUBROUTINE POSPOM(IP,IND1,IND2,KCUT,ISWAP,IREJ)
C***********************************************************************
C
C     registration of one cut pomeron (soft/semihard)
C
C     input:   IP      particle combination the pomeron belongs to
C              IND1,2  position of X values in COMMON /ABRSOF/
C                      1 corresponds to a valence-pomeron
C              KCUT    number of cut pomerons and reggeons 
C
C     output:  ISWAP   exchange of x values
C              IREJ    success/failure
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           DEPS   =  1.D-8)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
      COMMON /CUTOFF/ PTCUT(4),CUTMU(4),FPS(4),FPH(4),PSOMIN,XSOMIN
C
      PARAMETER (IEETAB=20)
      COMMON /XSETAB/ SIGTAB(4,70,IEETAB),SIGECM(4,IEETAB),ISIMAX
C
      COMMON /ABRSOF/ XS1(50),XS2(50),IJSI1(50),IJSI2(50),
     &                PSOFT1(4,50),PSOFT2(4,50)
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      COMMON/POMRES/IPOPOS(2,100),IPORES(100),IPOIX1,IPOIX2
C
      DIMENSION P1(4),P2(4),WGXHSD(2),WGX(6)
C
      IREJ = 0
      ISWAP = 0
      JM1 = NPOSP(1)
      JM2 = NPOSP(2)
      CMASS1 = SQRT(XS1(IND1)*XS2(IND2)*ECMP)
      CMASS2 = SQRT(XS1(IND1+1)*XS2(IND2+1)*ECMP)
      CMASS1 = MIN(CMASS1,CMASS2)
      CMASS2 = MAX(CMASS1,CUTMU(1))
C  debug output
      IF(IDEB(9).GE.20) THEN
        WRITE(6,'(1X,A,5I4)') 'POSPOM:DEBUG:IP,IND1,IND2,KCUT,IPOIX1',
     &    IP,IND1,IND2,KCUT,IPOIX1
        WRITE(6,'(1X,A,2I4,1P2E12.4)') 'MOTHER1/2,MASS1/2',JM1,JM2,
     &    CMASS1,CMASS2
      ENDIF
C  flavours
      IF(IND1.EQ.1) THEN
        CALL VALFLA(JM1,IFLA1,IFLA2,CMASS2)
        CALL QUASTA(IFLA1,IDHEP(JM1),1)
        CALL QUASTA(IFLA2,IDHEP(JM1),1)
      ELSE
        CALL SEAFLA(JM1,IFLA1,IFLA2,CMASS2)
        CALL QUASTA(IFLA1,IDHEP(JM1),3)
        CALL QUASTA(IFLA2,IDHEP(JM1),3)
      ENDIF
      IF(IND2.EQ.1) THEN
        CALL VALFLA(JM2,IFLB1,IFLB2,CMASS2)
        CALL QUASTA(IFLB1,IDHEP(JM2),1)
        CALL QUASTA(IFLB2,IDHEP(JM2),1)
      ELSE
        CALL SEAFLA(JM2,IFLB1,IFLB2,CMASS2)
        CALL QUASTA(IFLB1,IDHEP(JM2),3)
        CALL QUASTA(IFLB2,IDHEP(JM2),3)
      ENDIF
      DO 75 I=1,4
        P1(I) = PSOFT1(I,IND1)+PSOFT1(I,IND1+1)
        P2(I) = PSOFT2(I,IND2)+PSOFT2(I,IND2+1)
 75   CONTINUE
C*************************
C  pomeron resolved?
      IF((ISWMDL(14).GT.0).AND.(IPOIX1.GT.0)) THEN
        SSUB = (P1(4)+P2(4))**2-(P1(1)+P2(1))**2
     &        -(P1(2)+P2(2))**2-(P1(3)+P2(3))**2
        ESUB = SQRT(SSUB)
C  load cross sections from interpolation table
        IF(ESUB.LE.SIGECM(IP,1)) THEN
          I1 = 1
          I2 = 1
        ELSE IF(ESUB.LT.SIGECM(IP,ISIMAX)) THEN
          DO 50 I=2,ISIMAX
            IF(ESUB.LE.SIGECM(IP,I)) GOTO 200
 50       CONTINUE
 200      CONTINUE
          I1 = I-1
          I2 = I
        ELSE
          WRITE(6,'(/1X,A,2E12.3)')
     &      'POSPOM:WARNING:too high energy',ESUB,SIGECM(IP,ISIMAX)
          CALL POPREV(-1)
          I1 = ISIMAX
          I2 = ISIMAX
        ENDIF
        FAC2=0.D0
        IF(I1.NE.I2) FAC2=LOG(ESUB/SIGECM(IP,I1))
     &                    /LOG(SIGECM(IP,I2)/SIGECM(IP,I1))
        FAC1=1.D0-FAC2
C  calculate weights
        WGXPOM = FAC2*SIGTAB(IP,69,I2)+FAC1*SIGTAB(IP,69,I1)
        WGXHSD(1) = FAC2*SIGTAB(IP,32,I2)+FAC1*SIGTAB(IP,32,I1)
        WGXHSD(2) = FAC2*SIGTAB(IP,33,I2)+FAC1*SIGTAB(IP,33,I1)
        WGXHDD = FAC2*SIGTAB(IP,35,I2)+FAC1*SIGTAB(IP,35,I1)
        WGXCDF = FAC2*SIGTAB(IP,36,I2)+FAC1*SIGTAB(IP,36,I1)
        WGX(1) = WGXPOM-3.D0*(WGXHSD(1)+WGXHSD(2)+WGXHDD)+15.D0*WGXCDF
        WGX(2) = WGXCDF
        WGX(3) = WGXHSD(1)
        WGX(4) = WGXHSD(2)
        WGX(5) = WGXHDD
        WGX(6) = 2.D0*(WGXHSD(1)+WGXHSD(2)+WGXHDD)
        SUM  = WGX(1)+WGX(2)+WGX(3)+WGX(4)+WGX(5)+WGX(6)
C  selection loop
 205    CONTINUE
        XI = RNDM(SUM)*SUM
        I = 0
        SUM = 0.D0
 210    CONTINUE
          I = I+1
          SUM = SUM+WGX(I)
        IF((XI.GT.SUM).AND.(I.LT.6)) GOTO 210
          IF(I.NE.1) THEN
            ISAM = 4
            IF(I.EQ.6) ISAM = 8
            PACC = EXP(-PARMDL(8)*DBLE(ISAM*CUTMU(1))/ESUB)
*           IF(RNDM(SUM).GT.PACC) I=1
            IF(RNDM(SUM).GT.PACC) GOTO 205
          ENDIF
C  do not generate diffraction for events with only one cut
        IF((KCUT.EQ.1).AND.(I.LT.6)) I = 1
C  debug output
        IF(IDEB(9).GE.20) WRITE(6,'(1X,A,/1X,I2,1P7E11.3)') 
     &    'POSPOM:DEBUG:IPRO,ESUB,WGX(1-6)',I,ESUB,WGX
C  second scattering needed
        IF(I.GT.1) THEN
          CALL HACODE(IFLA1,IFLA2,IDHA1,IDUM)
          CALL HACODE(IFLB1,IFLB2,IDHA2,IDUM)
          IDPD1 = MPDGHA(IDHA1)
          IDPD2 = MPDGHA(IDHA2)
          CALL REGPAR(1,IDPD1,IDHA1,JM1,JM2,P1(1),P1(2),P1(3),P1(4),
     &      I,0,0,0,IPOS1,1)
          CALL REGPAR(1,IDPD2,IDHA2,JM2,JM1,P2(1),P2(2),P2(3),P2(4),
     &      I,0,0,0,IPOS1,1)
          IND1 = IND1+2
          IND2 = IND2+2
C  update index common
          IPOIX2 = IPOIX2+1
          IPORES(IPOIX2) = I+2
          IPOPOS(1,IPOIX2) = IPOS1-1
          IPOPOS(2,IPOIX2) = IPOS1
          RETURN
        ENDIF
      ENDIF
C*************************
C
 100  CONTINUE
      IF(ISWMDL(12).EQ.0) THEN
C  sample colors
        CALL SELCOL(0,0,ICA1,ICA2,ICB1,ICB2,1)
        CALL SELCOL(0,0,ICC1,ICC2,ICD1,ICD2,1)
C  valence quark labels
        IF(IND1.EQ.1) THEN
          ICA2 = 1
          ICD2 = 1
        ENDIF
        IF(IND2.EQ.1) THEN
          ICB2 = 1
          ICC2 = 1
        ENDIF
C  color connection
        IF(((ABS(IFLA1).GT.6).AND.(IFLA1.GT.0))
     &    .OR.((ABS(IFLA1).LE.6).AND.(IFLA1.LT.0)))
     &    CALL SWAPI(ICA1,ICD1)
        IF(((ABS(IFLB1).GT.6).AND.(IFLB1.LT.0))
     &    .OR.((ABS(IFLB1).LE.6).AND.(IFLB1.GT.0)))
     &    CALL SWAPI(ICB1,ICC1)
        ISWAP = 0
        IF(ICA1*ICB1.GT.0) THEN
          IF((IND1.NE.1).AND.(IND2.NE.1)) THEN
            IF(RNDM(CMASS1).GT.OHALF) THEN
              CALL SWAPI(IFLA1,IFLA2)
              CALL SWAPI(ICA1,ICD1)
            ELSE
              CALL SWAPI(IFLB1,IFLB2)
              CALL SWAPI(ICB1,ICC1)
            ENDIF
          ELSE IF(IND1.NE.1) THEN
            CALL SWAPI(IFLA1,IFLA2)
            CALL SWAPI(ICA1,ICD1)
          ELSE IF(IND2.NE.1) THEN
            CALL SWAPI(IFLB1,IFLB2)
            CALL SWAPI(ICB1,ICC1)
          ELSE IF((IFLA1.EQ.-IFLA2).AND.(IFLB1.EQ.-IFLB2)) THEN
            IF(RNDM(CMASS1).GT.OHALF) THEN
              CALL SWAPI(IFLA1,IFLA2)
              CALL SWAPI(ICA1,ICD1)
            ELSE
              CALL SWAPI(IFLB1,IFLB2)
              CALL SWAPI(ICB1,ICC1)
            ENDIF
          ELSE IF(IFLA1.EQ.-IFLA2) THEN
            CALL SWAPI(IFLA1,IFLA2)
            CALL SWAPI(ICA1,ICD1)
          ELSE IF(IFLB1.EQ.-IFLB2) THEN
            CALL SWAPI(IFLB1,IFLB2)
            CALL SWAPI(ICB1,ICC1)
          ELSE
            ISWAP = 1
*           WRITE(6,'(1X,A)') 'POSPOM:WARNING:chain end swap'
*           WRITE(6,'(5X,A,4I7)') 'flavors:',IFLA1,IFLA2,IFLB1,IFLB2
*           WRITE(6,'(5X,A,4I7)') 'colors :',ICA1,ICD1,ICB1,ICC1
*           CALL POPREV(-1)
          ENDIF
        ENDIF
C  registration
        CALL REGPAR(-1,IFLA1,0,JM1,JM2,PSOFT1(1,IND1),PSOFT1(2,IND1),
     &    PSOFT1(3,IND1),PSOFT1(4,IND1),I,0,ICA1,ICA2,IPOS1,1)
        IND1 = IND1+1
        CALL REGPAR(-1,IFLA2,0,JM1,JM2,PSOFT1(1,IND1),PSOFT1(2,IND1),
     &      PSOFT1(3,IND1),PSOFT1(4,IND1),I,0,ICD1,ICD2,IPOS,1)
        IND1 = IND1+1
        CALL REGPAR(-1,IFLB1,0,JM2,JM1,PSOFT2(1,IND2),PSOFT2(2,IND2),
     &      PSOFT2(3,IND2),PSOFT2(4,IND2),I,0,ICB1,ICB2,IPOS,1)
        IND2 = IND2+1
        CALL REGPAR(-1,IFLB2,0,JM2,JM1,PSOFT2(1,IND2),PSOFT2(2,IND2),
     &      PSOFT2(3,IND2),PSOFT2(4,IND2),I,0,ICC1,ICC2,IPOS2,1)
        IND2 = IND2+1
C  soft pt assignment
        IF(ISWMDL(18).EQ.0) THEN
          CALL PARTPT(0,IPOS1,IPOS2,PTCUT(IP),IREJ)
          IF(IREJ.NE.0) RETURN
        ENDIF
      ELSE
*       CALL POBFKL(P1,P2,IPART,IREJ)
*       IF(IREJ.NE.0) RETURN
      ENDIF
      END
