C
C
      SUBROUTINE QELAST(IPROC,JM1,JM2,IREJ)
C**********************************************************************
C
C     sampling of quasi elastic processes
C
C     input:   IPROC  2   purely elastic scattering
C              IPROC  3   q-ela. omega/omega/phi/pi+pi- production
C              IPROC  4   double pomeron scattering
C              IPROC  -1  initialization
C              IPROC  -2  output of statistics
C              JM1        index of initial particle 1
C              JM2        index of initial particle 2
C
C     output:  initial and final particles in COMMON HEPEVS involving
C              polarized resonances in COMMON HEPEVS and decay
C              products
C
C              IREJ    0  successful
C                      1  failure
C                     50  user rejection
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER(NTAB = 20,
     &          EPS  = 1.D-10,
     &          IONE = 1,
     &          ONE  = 1.D0,
     &          PIMASS = 0.13D0,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 1.D-10,
     &          ZERO = 0.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C  CM system of diffraction
      COMMON /DIFCMS/ ECMD,PCMD,PMASSD(2),PVIRTD(2),GAMBED(4),
     &                SIDD,CODD,SIFD,COFD,PDCMS(4,2),NPOSD(2)
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  cross sections
      COMMON /XSECTP/ SIGTOT,SIGELA,SIGVM(0:4,0:4),SIGINE,SIGDIR,
     &                SIGLSD(2),SIGHSD(2),SIGLDD,SIGHDD,SIGCDF,
     &                SIGPOM,SIGREG,SIGHAR,SIGTR1,SIGTR2,SIGLOO,SIGDPO,
     &                SIG1SO,SIG1HA,SLOEL,SLOVM(4,4),SIGGEN(4),
     &                FSUP(2)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      DIMENSION   P1(5),P(5,2),PMI(2),RMASS(2),IFL(2),IDPRO(4)
      CHARACTER*8 VMESA(0:4),VMESB(0:4),PANAME
      DIMENSION   ISAMVM(4,4)
      DATA IDPRO / 113,223,333,6666 /
      DATA VMESA / 'VMESON  ','RHO     ','OMEGA0  ','PHI     ',
     &             'PI+PI-  ' /
      DATA VMESB / 'VMESON  ','RHO     ','OMEGA0  ','PHI     ',
     &             'PI+PI-' /
C
C  sampling of quasi elastic processes
      IF((IPROC.EQ.2).OR.(IPROC.EQ.3)) THEN
        IREJ = 0
        NPOSD(1) = JM1
        NPOSD(2) = JM2
        DO 55 I=1,2
          PMI(I) = PHEP(5,NPOSD(I))
          IF(PMI(I).LT.0.1D0) PMI(I) = 0.765D0
 55     CONTINUE
C  get CM system
        P1(1) = PHEP(1,JM1)+PHEP(1,JM2)
        P1(2) = PHEP(2,JM1)+PHEP(2,JM2)
        P1(3) = PHEP(3,JM1)+PHEP(3,JM2)
        P1(4) = PHEP(4,JM1)+PHEP(4,JM2)
        SS = P1(4)**2-P1(1)**2-P1(2)**2-P1(3)**2
        ECMD = SQRT(SS)
C
        IF(ECMD.LE.PMI(1)+PMI(2)) THEN
          IF(IDEB(34).GE.3) WRITE(6,'(1X,A,I12,3E12.4)')
     &      'QELAST:WARNING:TOO SMALL MASS (EV,ECM,M1,M2)',KEVENT,
     &      ECMD,PMI
          IREJ = 5
          RETURN
        ENDIF
C
        DO 60 I=1,4
          GAMBED(I) = P1(I)/ECMD
 60     CONTINUE
        CALL ALTRA(GAMBED(4),-GAMBED(1),-GAMBED(2),-GAMBED(3),
     &           PHEP(1,NPOSD(1)),PHEP(2,NPOSD(1)),PHEP(3,NPOSD(1)),
     &           PHEP(4,NPOSD(1)),PTOT1,P1(1),P1(2),P1(3),P1(4))
C  rotation angles
        CODD = P1(3)/PTOT1
        SIDD = SQRT((ONE-CODD)*(ONE+CODD))
        COFD = ONE
        SIFD = ZERO
        IF(PTOT1*SIDD.GT.DEPS) THEN
          COFD = P1(1)/(SIDD*PTOT1)
          SIFD = P1(2)/(SIDD*PTOT1)
          ANORF= SQRT(COFD*COFD+SIFD*SIFD)
          COFD = COFD/ANORF
          SIFD = SIFD/ANORF
        ENDIF
C  get CM momentum
        AM12 = PMI(1)**2
        AM22 = PMI(2)**2
        PCMD = XLAMB(SS,AM12,AM22)/(2.D0*ECMD)
C
        ICALL = ICALL + 1
C  main rejection label
 50     CONTINUE
C  determine process and final particles
        IFL(1) = IDHEP(NPOSD(1))
        IFL(2) = IDHEP(NPOSD(2))
        IF(IPROC.EQ.3) THEN
          ITRY = 0
 100      CONTINUE
          ITRY = ITRY+1
          IF(ITRY.GT.50) THEN
            IF(IDEB(34).GE.3) WRITE(6,'(1X,A,I12,I5,E12.4)')
     &        'QELAST:WARNING:MASS REJECTION (EV,ITRY,ECM)',KEVENT,
     &        ITRY,ECMD
            IREJ = 5
            RETURN
          ENDIF
          XI = RNDM(PCMD)*SIGVM(0,0)-DEPS
          DO 110 I=1,4
            DO 120 J=1,4
              XI = XI-SIGVM(I,J)
              IF(XI.LE.ZERO) GOTO 130
 120        CONTINUE
 110      CONTINUE
 130      CONTINUE
          IF(IFL(1).EQ.22) IFL(1) = IDPRO(I)
          IF(IFL(2).EQ.22) IFL(2) = IDPRO(J)
          ISAMVM(I,J) = ISAMVM(I,J)+1
          ISAMQE = ISAMQE+1
C  sample new masses
          CALL SAMASS(IFL(1),RMASS(1))
          CALL SAMASS(IFL(2),RMASS(2))
          IF(RMASS(1)+RMASS(2).GE.ECMD) GOTO 100
        ELSE IF(IPROC.EQ.2) THEN
          I = 0
          J = 0
          ISAMEL = ISAMEL+1
          RMASS(1) = POPAMA(NPOSD(1),2)
          RMASS(2) = POPAMA(NPOSD(2),2)
        ELSE
          WRITE(6,'(/1X,A,I6)') 'QELAST:ERROR:INVALID IPROC',IPROC
          CALL POABRT
        ENDIF
C  sample momentum transfer
        CALL SADSDT(I,J,TT)
        IF(IDEB(34).GE.5) WRITE(6,'(1X,A,2I6,I3,3E11.3)')
     &    'QELAST:DEBUG:IF1,2,T,RM1,RM2',IFL,IPROC,TT,RMASS
C  calculate new momenta
        CALL DIFKIN(RMASS(1),RMASS(2),TT,P(1,1),P(1,2),IREJ)
        IF(IREJ.NE.0) GOTO 50
C  comment line for elastic/quasi-elastic scattering
        CALL REGPAR(35,IPROC,0,NPOSD(1),NPOSD(2),RMASS(1),RMASS(2),TT,
     &    ECMD,IFL(1),IFL(2),IDHEP(NPOSD(1)),IDHEP(NPOSD(2)),ICPOS,1)
C
        I1 = NHEP+1
C  fill COMMON HEPEVS
        DO 200 I=1,2
          K = 3-I
          IF(IFL(I).EQ.6666) THEN
C  pi+/pi- isotropic background
            CALL REGPAR(1,113,33,NPOSD(I),NPOSD(K),P(1,I),P(2,I),
     &        P(3,I),P(4,I),0,0,0,0,IPOS,1)
            ICOLOR(I,ICPOS) = IPOS
            CALL SDECAY(IPOS,0,-2)
          ELSE
C  registration
            CALL REGPAR(1,IFL(I),0,NPOSD(I),NPOSD(K),P(1,I),P(2,I),
     &        P(3,I),P(4,I),0,0,0,0,IPOS,1)
            ICOLOR(I,ICPOS) = IPOS
          ENDIF
 200    CONTINUE
        I2 = NHEP
C  search for vector-mesons
        DO 300 I=I1,I2
C  decay according to polarization
          IF(IDHEP(JMOHEP(1,I)).EQ.22) THEN
            ISP = IPAMDL(3)
            IF(ISWMDL(21).GE.1) ISP = IPAMDL(4)
            CALL SDECAY(I,ISP,2)
          ENDIF
 300    CONTINUE
        I2 = NHEP
C  back transformation
        CALL LTRHEP(I1,I2,CODD,SIDD,COFD,SIFD,GAMBED(4),GAMBED(1),
     &              GAMBED(2),GAMBED(3))
C
C  initialization of tables
      ELSE IF(IPROC.EQ.-1) THEN
        WRITE(6,'(/,1X,A,/,1X,A)')
     &       'INITIALIZATION OF ELASTIC/QUASI-ELASTIC PROCESSES:',
     &       '=================================================='
        DO 10 I=1,4
          DO 20 J=1,4
            ISAMVM(I,J) = 0
 20       CONTINUE
 10     CONTINUE
        ISAMEL = 0
        ISAMQE = 0
        IF(IDHEP(JM1).NE.22) THEN
          VMESA(1) = PANAME(IMPART(JM1),0)
        ELSE
          VMESA(1) = 'RHO     '
        ENDIF
        IF(IDHEP(JM2).NE.22) THEN
          VMESB(1) = PANAME(IMPART(JM2),0)
        ELSE
          VMESB(1) = 'RHO     '
        ENDIF
        CALL SAMASS(-1,RMASS(1))
        CALL SADSDT(-1,0,TT)
        ICALL = 0
C
C  output of statistics
      ELSE IF(IPROC.EQ.-2) THEN
        WRITE(6,'(/,1X,A,I10/,1X,A)')
     &       'STATISTICS OF ELASTIC/QUASI-ELASTIC PROCESSES:',ICALL,
     &       '=============================================='
        IF(ICALL.LT.10) RETURN
        WRITE(6,'(1X,A,I10)')
     &    'SAMPLED ELASTIC PROCESSES:',ISAMEL
        WRITE(6,'(1X,A,I10)')
     &    'SAMPLED QUASI-ELASTIC VECTORMESON PRODUCTION:',ISAMQE
        WRITE(6,'(15X,4(4X,A8))') (VMESB(I),I=1,4)
        DO 30 I=1,4
          WRITE(6,'(5X,A8,4I12)') VMESA(I),(ISAMVM(I,K),K=1,4)
 30     CONTINUE
        CALL SAMASS(-2,RMASS(1))
        CALL SADSDT(-2,0,TT)
      ELSE
        WRITE(6,'(/1X,A,I3)') 'QELAST:ERROR:UNKNOWN PROCESS ',IPROC
        CALL POABRT
      ENDIF
      END
