C
C     auxiliary routines for PHOJET histogramming
C             (reference system independent part)
C
C
C
      SUBROUTINE UAJETF(ETAL1,ETAL2,NETA,NPHI,ETINI,ETMIN,ETBIN,ETMAX,
     &                  RJET,NJETS,FJETS)
C********************************************************************
C
C  Cone jet finder algorithm
C
C  input:  ETAL1  lower pseudorapidity limit
C          ETAL2  upper pseudorapidity limit
C          NETA   number of pseudorapidity bins in ETA-PHI space (MAX=35)
C          NPHI   number of PHI  bins in ETA-PHI space (MAX=35)
C          ETINI  min transverse energy of initiator
C          ETMIN  Etjet min
C          ETBIN  bin size for Etjet distribution
C          ETMAX  Etjet max
C          RJET   cone size
C
C  output: NJETS        number of jets
C          FJETS        FJETS(10,3)
C          FJETS(I,1)   Etjet
C          FJETS(I,2)   ETA jet (pseudorapidity)
C          FJETS(I,3)   azimuthal angle of jet axes (0 - 2*PI)
C          FJETS(I,4-7) 4-momentum vector of jet (7: energy)
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( MAXCE = 35,
     &            PI    = 3.14159265359D0,
     &            TWOPI = 6.28318530718D0,
     &            ZERO  = 0.D0,
     &            ONE   = 1.D0,
     &            TWO   = 2.D0 )
      DIMENSION  FJETS(10,7),CELL(MAXCE,MAXCE),ETAC(MAXCE),PHIC(MAXCE)
      DIMENSION  LOCK(MAXCE,MAXCE)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      NJETS = 0
      IF((NETA.GT.MAXCE).OR.(NPHI.GT.MAXCE)) THEN
        WRITE(6,'(1X,A,4I4)')
     &    'UAJETF:ERROR:INVALID GRID (NE/NEMAX,NP/NPMAX)',NETA,MAXCE,
     &    NPHI,MAXCE
        RETURN
      ENDIF
C ----------------------------------------
C     Internal 'jet' steering:
C
C         1. Ecut for the cell Et energy |Et cell| > ETCUT
C
      ETCUT = 0.05D0
C ----------------------------------------
      BINETA = (ETAL2-ETAL1)/DBLE(NETA)
      BINPHI = TWOPI/DBLE(NPHI)
      BINET2 = BINETA/TWO
      BINPH2 = BINPHI/TWO
      RJET2  = RJET**2
C
      MAXD = MAX(NETA,NPHI)
      DO 12 I=1,MAXD
        DO 11 K=1,MAXD
          CELL(I,K) = ZERO
 11     CONTINUE
 12   CONTINUE
C
C
C     Fill CELL() here
C==================================================================
C
      DO 100 ITRK = 1,NHEP
        IF(ISTHEP(ITRK).EQ.1) THEN
          ICOD = IDHEP(ITRK)
C  remove neutrinos
          IF((ICOD.EQ.12).OR.(ICOD.EQ.14).OR.(ICOD.EQ.16)) GOTO 98
          PX   = PHEP(1,ITRK)
          PY   = PHEP(2,ITRK)
          PZ   = PHEP(3,ITRK)
          E    = PHEP(4,ITRK)
          PXY= PX**2 + PY**2
          IF(PXY.LT.1.D-12) GOTO 98
          P  = SQRT(PXY + PZ**2)
          IF(ABS(P).LE.1.D-6) GOTO 98
          THETA = ACOS(PZ/P)
          IF((THETA.LT.1.D-8).OR.(PI-THETA.LT.1.D-8)) GOTO 98
          ETA = -LOG(TAN(THETA/TWO))
          IF((ETA.LE.ETAL1).OR.(ETA.GE.ETAL2)) GOTO 98
          PHI   = ATAN2(PY,PX)
          IF(PHI.LT.0.D0) PHI = TWOPI + PHI
          ET = E*SIN(THETA)
          IETA = 1 + INT((ETA-ETAL1)/BINETA)
          IF(IETA.GT.NETA) GOTO 98
          IPHI = 1 + INT(PHI/BINPHI)
          IF(IPHI.GT.NPHI) GOTO 98
          CELL(IETA,IPHI) = ET + CELL(IETA,IPHI)
        ENDIF
 98     CONTINUE
 100  CONTINUE
C
C  remove "invisible" cells
      ETSUM = ZERO
      DO 57 I=1,NETA
        DO 54 J=1,NPHI
          IF(CELL(I,J).LT.ETCUT) THEN
            LOCK(I,J) = 2
          ELSE
            LOCK(I,J) = 0
            ETSUM = ETSUM + CELL(I,J)
          ENDIF
 54     CONTINUE
 57   CONTINUE
*====================================================================
C
      IF(ETSUM.LT.ETMIN) RETURN
C
C  make search index
      DO 15 I=1,MAXD
        ETAC(I) = ETAL1 + BINETA*DBLE(I) - BINET2
        PHIC(I) = BINPHI*DBLE(I) - BINPH2
 15   CONTINUE
*
 9    CONTINUE
****************************************************************
      IETMAX = NINT(ETMAX)
      IETMIN = NINT(ETMIN)
      IETBIN = NINT(ETBIN)
      ETCON = -ONE
*
*  loop over Etjet distribution
      DO 80 IETM = IETMAX-IETBIN,IETMIN,-IETBIN
C
        ETMIN1 = DBLE(IETM)
C  jets possible?
        IF((ETCON.GT.ZERO).AND.(ETCON.LT.ETMIN1)) GOTO 75
        ETCON = -ONE
        IF(ETSUM.LT.ETMIN1) GOTO 75
C
C  -------------- begin of main loop ----------------
C
10      CONTINUE
C
C  find initiator cell (cell with maximum of et)
          ET1 = ETINI
          JET1 = -9999
          DO 17 I=1,NETA
            DO 14 J=1,NPHI
              IF((LOCK(I,J).EQ.0).AND.(ET1.LT.CELL(I,J))) THEN
                ET1 = CELL(I,J)
                IET1 = I
                JET1 = J
              ENDIF
 14         CONTINUE
 17       CONTINUE
C
C  no cell found: unlock initiator cells
          IF(JET1.EQ.-9999) THEN
            DO 47 I=1,NETA
              DO 44 J=1,NPHI
                IF(LOCK(I,J).EQ.1) LOCK(I,J) = 0
 44           CONTINUE
 47         CONTINUE
            GOTO 80
          ENDIF
C
          LOCK(IET1,JET1) = 1
C  jet axis
          ETAET1 = ETAC(IET1)
          PHIET1 = PHIC(JET1)
*
*       Collect cells
*
*
          ETJET = ZERO
          DO 24 J=1,NPHI
            DPHI = ABS(PHIC(J)-PHIET1)
            IF(DPHI.GT.PI) DPHI = TWOPI - DPHI
            DRPHI2 = RJET2-DPHI*DPHI
            IF(DRPHI2.LT.ZERO) GOTO 25
            DRPHI = SQRT(DRPHI2)
            NBET = DRPHI/BINETA+1
            IMIN = MAX(1,IET1-NBET)
            IMAX = MIN(NETA,IET1+NBET)
            DO 27 I=IMIN,IMAX
              IF(LOCK(I,J).GT.1) GOTO 23
              IF(ABS(ETAC(I)-ETAET1).GT.DRPHI) GOTO 23
              ETJET = ETJET + CELL(I,J)
 23           CONTINUE
 27         CONTINUE
 25         CONTINUE
 24       CONTINUE
        IF(ETJET.LT.ETMIN1) THEN
          ETCON = MAX(ETCON,ETJET)
          GOTO 10
        ENDIF
C
C  -------------- end of main loop ----------------
*
*        Correct Eta and Phi
*
        ETAJET = ZERO
        ETPOS  = ZERO
        XJET = ZERO
        YJET = ZERO
        ZJET = ZERO
        EJET = ZERO
*
        DO 34 J=1,NPHI
          PHI  = PHIC(J)
          DPHI = ABS(PHI-PHIET1)
          IF(DPHI.GT.PI) DPHI = TWOPI - DPHI
          DRPHI2 = RJET2-DPHI*DPHI
          IF(DRPHI2.LT.ZERO) GOTO 33
          DRPHI = SQRT(DRPHI2)
          NBET = DRPHI/BINETA+1
          IMIN = MAX(1,IET1-NBET)
          IMAX = MIN(NETA,IET1+NBET)
          DO 37 I=IMIN,IMAX
            IF(LOCK(I,J).GT.1)   GOTO 35
            ETA = ETAC(I)
            IF(ABS(ETA-ETAET1).GT.DRPHI) GOTO 35
            LOCK(I,J) = 2
            EADD = CELL(I,J)
            ETPOS = ETPOS + EADD
            THETA = TWO*ATAN(EXP(-ETA))
            EJ = EADD/SIN(THETA)
            XJ = EADD*COS(PHI)
            YJ = EADD*SIN(PHI)
            ZJ = EJ*COS(THETA)
            ETAJET = ETAJET + ETA*EADD
            XJET = XJET + XJ
            YJET = YJET + YJ
            ZJET = ZJET + ZJ
            EJET = EJET + EJ
 35         CONTINUE
 37       CONTINUE
 33       CONTINUE
 34     CONTINUE
C
        IF(ETPOS.GT.ZERO) THEN
          ETSUM = ETSUM-ETPOS
          ETAJET = ETAJET/ETPOS
          PHIJET = ATAN2(YJET,XJET)
          IF(PHIJET.LT.ZERO) PHIJET = TWOPI + PHIJET
        ELSE
          WRITE(6,'(1X,A)') 'UAJETF:WARNING:TOO MANY NEGATIVE CELLS'
          GOTO 10
        ENDIF
C
        DPHI = PHIJET-PHIET1
        IF(DPHI.LT.-PI) DPHI = - DPHI- TWOPI
        IF(DPHI.GT.PI)  DPHI = TWOPI - DPHI
        IF(NJETS.LT.10) THEN
          NJETS = NJETS + 1
          FJETS(NJETS,1) = ETJET
          FJETS(NJETS,2) = ETAJET
          FJETS(NJETS,3) = PHIJET
          FJETS(NJETS,4) = XJET
          FJETS(NJETS,5) = YJET
          FJETS(NJETS,6) = ZJET
          FJETS(NJETS,7) = EJET
          GOTO 10
        ELSE
          WRITE(6,'(1X,A,I5)')
     &      'UAJETF:WARNING:TOO MANY JETS FOUND',NJETS
          GOTO 99
        ENDIF
C
 75     CONTINUE
 80   CONTINUE
C
C  final output
 99   CONTINUE
      RETURN
      END
