/** 
 * Example of running KtJet in exclusive mode 
 * e+e- events (1)
 * Angular distance scheme (1)
 * Recombination scheme pT (2)
 * 4 vectors are read in from a txt file in format of
 * px py pz E


 @author J.Couchman
*/

#include <iostream>
#include <fstream>
#include <string>

/** Need to include these KtJet Headers */
#include "KtJet/KtEvent.h"
#include "KtJet/KtLorentzVector.h"

int main(){
  /** Open input file */
  const char* filename = "inputParticles2.dat";
  ifstream infile(filename);
  if (!infile) {
    std::cerr << "Unable to open file!!!!" << filename << std::endl;
    return -1;
  }
  /** Make a std::vector of KtJet::KtLorentzVectors which will be the 
      input particles */
  std::vector<KtJet::KtLorentzVector> avec;
  /** read in 4-vectors and fill std::vector */
  double value[4];
  while(infile >> value[0]){
    infile >> value[1];
    infile >> value[2];
    infile >> value[3];
    KtJet::KtLorentzVector p(value[0],value[1],value[2],value[3]);
    avec.push_back(p);
  }

  /** set KtEvent flags */
  int type  = 1; // PP
  int angle = 1; //Angular
  int recom = 2; // E

  /** Construct the KtEvent object */
  KtJet::KtEvent ev(avec,type,angle,recom);

  /** Print out total energy of event */
  std::cout << "Total energy of event: " << ev.getETot() << std::endl;

  /** Force event into 4 jets */
  ev.findJetsN(4);
 
 /** Retrieve four jets */
  std::vector<KtJet::KtLorentzVector> jets = ev.getJets();

  /** Print out jets 4-momentum and Pt of four jets*/
  std::vector<KtJet::KtLorentzVector>::const_iterator itr = jets.begin();
  for( ; itr != jets.end() ; ++itr) {
    std::cout << "Jets 4 vector: " 
	      << (*itr).px() << " " 
	      << (*itr).py() << " " 
	      << (*itr).pz() << " " 
	      << (*itr).e() << std::endl;
    std::cout << "Jets Pt: " << (*itr).perp() << std::endl; 
  }

  /** Force event into jets with y-cut = 0.001 */
  double ycut = 0.001;
  ev.findJetsY(ycut);

 /** Print out how many jets are constructed*/
  std::cout << "number of Jets with y-cut set to " 
	    << ycut << " is " << ev.getNJets() << std::endl; 
  return 0;
}

